package java.lang;

import sun.text.Normalizer;
import java.text.BreakIterator;
import java.util.Iterator;
import java.util.Locale;
import java.util.HashSet;
import java.util.Hashtable;

final class ConditionalSpecialCasing
{
    static final int FINAL_CASED = 1;
    static final int AFTER_SOFT_DOTTED = 2;
    static final int MORE_ABOVE = 3;
    static final int AFTER_I = 4;
    static final int NOT_BEFORE_DOT = 5;
    static final int COMBINING_CLASS_ABOVE = 230;
    static ConditionalSpecialCasing.Entry[] entry;
    static Hashtable<Integer, HashSet<ConditionalSpecialCasing.Entry>> entryTable;
    
    ConditionalSpecialCasing() {
    }
    
    static int toLowerCaseEx(String s, int n, Locale locale) {
        char[] lookUpTable = lookUpTable(s, n, locale, true);
        if (lookUpTable != null) {
            if (lookUpTable.length == 1) {
                return lookUpTable[0];
            }
            return -1;
        }
        else {
            return Character.toLowerCase(s.codePointAt(n));
        }
    }
    
    static int toUpperCaseEx(String s, int n, Locale locale) {
        char[] lookUpTable = lookUpTable(s, n, locale, false);
        if (lookUpTable != null) {
            if (lookUpTable.length == 1) {
                return lookUpTable[0];
            }
            return -1;
        }
        else {
            return Character.toUpperCaseEx(s.codePointAt(n));
        }
    }
    
    static char[] toLowerCaseCharArray(String s, int n, Locale locale) {
        return lookUpTable(s, n, locale, true);
    }
    
    static char[] toUpperCaseCharArray(String s, int n, Locale locale) {
        char[] lookUpTable = lookUpTable(s, n, locale, false);
        if (lookUpTable != null) {
            return lookUpTable;
        }
        return Character.toUpperCaseCharArray(s.codePointAt(n));
    }
    
    private static char[] lookUpTable(String s, int n, Locale locale, boolean b) {
        HashSet<Object> set = (HashSet)ConditionalSpecialCasing.entryTable.get(new Integer(s.codePointAt(n)));
        char[] array = null;
        if (set != null) {
            Iterator<Object> iterator = set.iterator();
            String language = locale.getLanguage();
            while (iterator.hasNext()) {
                ConditionalSpecialCasing.Entry entry = (ConditionalSpecialCasing.Entry)iterator.next();
                String language2 = entry.getLanguage();
                if ((language2 == null || language2.equals(language)) && isConditionMet(s, n, locale, entry.getCondition())) {
                    array = (b ? entry.getLowerCase() : entry.getUpperCase());
                    if (language2 != null) {
                        break;
                    }
                    continue;
                }
                else {
                    continue;
                }
            }
        }
        return array;
    }
    
    private static boolean isConditionMet(String s, int n, Locale locale, int n2) {
        switch (n2) {
            case 1: {
                return isFinalCased(s, n, locale);
            }
            case 2: {
                return isAfterSoftDotted(s, n);
            }
            case 3: {
                return isMoreAbove(s, n);
            }
            case 4: {
                return isAfterI(s, n);
            }
            case 5: {
                return !isBeforeDot(s, n);
            }
            default: {
                return true;
            }
        }
    }
    
    private static boolean isFinalCased(String text, int offset, Locale locale) {
        BreakIterator wordInstance = BreakIterator.getWordInstance(locale);
        wordInstance.setText(text);
        while (offset >= 0 && !wordInstance.isBoundary(offset)) {
            int codePointBefore = text.codePointBefore(offset);
            if (isCased(codePointBefore)) {
                int length = text.length();
                int offset2 = offset + Character.charCount(text.codePointAt(offset));
                while (offset2 < length && !wordInstance.isBoundary(offset2)) {
                    int codePoint = text.codePointAt(offset2);
                    if (isCased(codePoint)) {
                        return false;
                    }
                    offset2 += Character.charCount(codePoint);
                }
                return true;
            }
            offset -= Character.charCount(codePointBefore);
        }
        return false;
    }
    
    private static boolean isAfterI(String s, int i) {
        while (i > 0) {
            int codePointBefore = s.codePointBefore(i);
            if (codePointBefore == 73) {
                return true;
            }
            int combiningClass = Normalizer.getCombiningClass(codePointBefore);
            if (combiningClass == 0 || combiningClass == 230) {
                return false;
            }
            i -= Character.charCount(codePointBefore);
        }
        return false;
    }
    
    private static boolean isAfterSoftDotted(String s, int i) {
        while (i > 0) {
            int codePointBefore = s.codePointBefore(i);
            if (isSoftDotted(codePointBefore)) {
                return true;
            }
            int combiningClass = Normalizer.getCombiningClass(codePointBefore);
            if (combiningClass == 0 || combiningClass == 230) {
                return false;
            }
            i -= Character.charCount(codePointBefore);
        }
        return false;
    }
    
    private static boolean isMoreAbove(String s, int n) {
        int length = s.length();
        int i = n + Character.charCount(s.codePointAt(n));
        while (i < length) {
            int codePoint = s.codePointAt(i);
            int combiningClass = Normalizer.getCombiningClass(codePoint);
            if (combiningClass == 230) {
                return true;
            }
            if (combiningClass == 0) {
                return false;
            }
            i += Character.charCount(codePoint);
        }
        return false;
    }
    
    private static boolean isBeforeDot(String s, int n) {
        int length = s.length();
        int i = n + Character.charCount(s.codePointAt(n));
        while (i < length) {
            int codePoint = s.codePointAt(i);
            if (codePoint == 775) {
                return true;
            }
            int combiningClass = Normalizer.getCombiningClass(codePoint);
            if (combiningClass == 0 || combiningClass == 230) {
                return false;
            }
            i += Character.charCount(codePoint);
        }
        return false;
    }
    
    private static boolean isCased(int n) {
        int type = Character.getType(n);
        return type == 2 || type == 1 || type == 3 || (n >= 688 && n <= 696) || (n >= 704 && n <= 705) || (n >= 736 && n <= 740) || n == 837 || n == 890 || (n >= 7468 && n <= 7521) || (n >= 8544 && n <= 8575) || (n >= 9398 && n <= 9449);
    }
    
    private static boolean isSoftDotted(int n) {
        switch (n) {
            case 105:
            case 106:
            case 303:
            case 616:
            case 1110:
            case 1112:
            case 7522:
            case 7725:
            case 7883:
            case 8305: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    
    static {
        ConditionalSpecialCasing.entry = new ConditionalSpecialCasing.Entry[] { new ConditionalSpecialCasing.Entry(931, new char[] { 'ς' }, new char[] { 'Σ' }, null, 1), new ConditionalSpecialCasing.Entry(304, new char[] { 'i', '̇' }, new char[] { 'İ' }, null, 0), new ConditionalSpecialCasing.Entry(775, new char[] { '̇' }, new char[0], "lt", 2), new ConditionalSpecialCasing.Entry(73, new char[] { 'i', '̇' }, new char[] { 'I' }, "lt", 3), new ConditionalSpecialCasing.Entry(74, new char[] { 'j', '̇' }, new char[] { 'J' }, "lt", 3), new ConditionalSpecialCasing.Entry(302, new char[] { 'į', '̇' }, new char[] { 'Į' }, "lt", 3), new ConditionalSpecialCasing.Entry(204, new char[] { 'i', '̇', '̀' }, new char[] { 'Ì' }, "lt", 0), new ConditionalSpecialCasing.Entry(205, new char[] { 'i', '̇', '́' }, new char[] { 'Í' }, "lt", 0), new ConditionalSpecialCasing.Entry(296, new char[] { 'i', '̇', '̃' }, new char[] { 'Ĩ' }, "lt", 0), new ConditionalSpecialCasing.Entry(304, new char[] { 'i' }, new char[] { 'İ' }, "tr", 0), new ConditionalSpecialCasing.Entry(304, new char[] { 'i' }, new char[] { 'İ' }, "az", 0), new ConditionalSpecialCasing.Entry(775, new char[0], new char[] { '̇' }, "tr", 4), new ConditionalSpecialCasing.Entry(775, new char[0], new char[] { '̇' }, "az", 4), new ConditionalSpecialCasing.Entry(73, new char[] { 'ı' }, new char[] { 'I' }, "tr", 5), new ConditionalSpecialCasing.Entry(73, new char[] { 'ı' }, new char[] { 'I' }, "az", 5), new ConditionalSpecialCasing.Entry(105, new char[] { 'i' }, new char[] { 'İ' }, "tr", 0), new ConditionalSpecialCasing.Entry(105, new char[] { 'i' }, new char[] { 'İ' }, "az", 0) };
        ConditionalSpecialCasing.entryTable = new Hashtable();
        for (int i = 0; i < ConditionalSpecialCasing.entry.length; ++i) {
            ConditionalSpecialCasing.Entry e = ConditionalSpecialCasing.entry[i];
            Integer o = new Integer(e.getCodePoint());
            HashSet<ConditionalSpecialCasing.Entry> value = (HashSet<ConditionalSpecialCasing.Entry>)ConditionalSpecialCasing.entryTable.get(o);
            if (value == null) {
                value = new HashSet();
            }
            value.add(e);
            ConditionalSpecialCasing.entryTable.put(o, value);
        }
    }
    
    static class Entry
    {
        int ch;
        char[] lower;
        char[] upper;
        String lang;
        int condition;
        
        Entry(int ch, char[] lower, char[] upper, String lang, int condition) {
            this.ch = ch;
            this.lower = lower;
            this.upper = upper;
            this.lang = lang;
            this.condition = condition;
        }
        
        int getCodePoint() {
            return this.ch;
        }
        
        char[] getLowerCase() {
            return this.lower;
        }
        
        char[] getUpperCase() {
            return this.upper;
        }
        
        String getLanguage() {
            return this.lang;
        }
        
        int getCondition() {
            return this.condition;
        }
    }
}
